@@ -1174,6 +1174,7 @@ namespace Google.GData.Client.LiveTests
                 newCalendar.Summary.Text = "some unique summary" + Guid.NewGuid().ToString();
                 newCalendar.TimeZone = "America/Los_Angeles";
                 newCalendar.Hidden = false;
+                newCalendar.Selected = true;
                 newCalendar.Color = "#2952A3";
                 newCalendar.Location = new Where("", "", "Test City");
 
@@ -1190,6 +1191,7 @@ namespace Google.GData.Client.LiveTests
                 Assert.AreEqual(newCalendar.Location.ValueString, createdCalendar.Location.ValueString, "Where should be equal");
 
                 createdCalendar.Title.Text = "renamed calendar" + Guid.NewGuid().ToString();
+                createdCalendar.Hidden = true;
                 CalendarEntry updatedCalendar = (CalendarEntry) createdCalendar.Update();
 
                 Assert.AreEqual(createdCalendar.Title.Text, updatedCalendar.Title.Text, "entry should have been updated");
@@ -1666,9 +1668,13 @@ namespace Google.GData.Client.LiveTests
         }
         /////////////////////////////////////////////////////////////////////////////
     } /////////////////////////////////////////////////////////////////////////////
+
+
 }
 
 
 
 
 
+
+
