@@ -726,7 +726,14 @@ namespace Google.YouTube
                 if (this.YouTubeEntry != null &&
                     this.YouTubeEntry.Rating != null)
                 {
-                    return this.YouTubeEntry.Rating.Value;
+                    try
+                    {
+                        return this.YouTubeEntry.Rating.Value;
+                    }
+                    catch (FormatException e)
+                    {
+                        return -1;
+                    }
                 }
                 return -1;
             }
@@ -814,6 +821,30 @@ namespace Google.YouTube
             }
         }
 
+        /// <summary>
+        /// boolean property shortcut to set the mediagroup/yt:private element. Setting this to true 
+        /// adds the element, if not already there (otherwise nothing happens)
+        /// setting this to false, removes it
+        /// </summary>
+        /// <returns></returns>
+        public bool Private
+        {
+            get
+            {
+                if (this.YouTubeEntry != null)
+                {
+                    return this.YouTubeEntry.Private;
+                }
+                return false;
+            }
+            set
+            {
+                EnsureInnerObject();
+                this.YouTubeEntry.Private = value;
+            }
+        }
+
+
 
 
         /// <summary>
@@ -892,19 +923,16 @@ namespace Google.YouTube
     /// <returns></returns>
     public class YouTubeRequestSettings : RequestSettings
     {
-        private string clientID;
         private string developerKey;
 
         /// <summary>
         /// A constructor for a readonly scenario.
         /// </summary>
         /// <param name="applicationName">The name of the application</param>
-        /// <param name="client">the client ID to use</param>
         /// <param name="developerKey">the developer key to use</param>
         /// <returns></returns>
-        public YouTubeRequestSettings(string applicationName, string client, string developerKey) : base(applicationName)
+        public YouTubeRequestSettings(string applicationName, string developerKey) : base(applicationName)
         {
-            this.clientID = client;
             this.developerKey = developerKey;
         }
 
@@ -912,15 +940,13 @@ namespace Google.YouTube
         /// A constructor for a client login scenario
         /// </summary>
         /// <param name="applicationName">The name of the application</param>
-        /// <param name="client">the client ID to use</param>
         /// <param name="developerKey">the developer key to use</param>
         /// <param name="userName">the username</param>
         /// <param name="passWord">the password</param>
         /// <returns></returns>
-        public YouTubeRequestSettings(string applicationName, string client, string developerKey, string userName, string passWord)  
+        public YouTubeRequestSettings(string applicationName, string developerKey, string userName, string passWord)  
                     : base(applicationName, userName, passWord)
         {
-            this.clientID = client;
             this.developerKey = developerKey;
         }
 
@@ -928,29 +954,40 @@ namespace Google.YouTube
         /// a constructor for a web application authentication scenario        
         /// </summary>
         /// <param name="applicationName">The name of the application</param>
-        /// <param name="client">the client ID to use</param>
         /// <param name="developerKey">the developer key to use</param>
         /// <param name="authSubToken">the authentication token</param>
         /// <returns></returns>
-        public YouTubeRequestSettings(string applicationName, string client, string developerKey, string authSubToken)  
+        public YouTubeRequestSettings(string applicationName, string developerKey, string authSubToken)  
                     : base(applicationName, authSubToken)
         {
-            this.clientID = client;
             this.developerKey = developerKey;
         }
 
         /// <summary>
-        /// returns the client ID
+        ///  a constructor for OpenAuthentication login use cases using 2 or 3 legged oAuth
         /// </summary>
+        /// <param name="applicationName">The name of the application</param>
+        /// <param name="developerKey">the developer key to use</param>
+        /// <param name="consumerKey">the consumerKey to use</param>
+        /// <param name="consumerSecret">the consumerSecret to use</param>
+        /// <param name="token">The token to be used</param>
+        /// <param name="tokenSecret">The tokenSecret to be used</param>
+        /// <param name="user">the username to use</param>
+        /// <param name="domain">the domain to use</param>
         /// <returns></returns>
-        public string Client
+        public YouTubeRequestSettings(string applicationName, 
+                               string developerKey,
+                               string consumerKey, string consumerSecret, 
+                               string token, string tokenSecret,
+                               string user, string domain) : base(applicationName, consumerKey, consumerSecret, 
+                                                            token, tokenSecret, user, domain)
         {
-            get
-            {
-                return this.clientID;
-            }
+            this.developerKey = developerKey;
         }
 
+
+        
+
         /// <summary>
         /// returns the developer key
         /// </summary>
@@ -1009,9 +1046,9 @@ namespace Google.YouTube
         /// <param name="settings"></param>
         public YouTubeRequest(YouTubeRequestSettings settings) : base(settings)
         {
-            if (settings.Client != null && settings.DeveloperKey != null)
+            if (settings.DeveloperKey != null)
             {
-                this.Service = new YouTubeService(settings.Application, settings.Client, settings.DeveloperKey);
+                this.Service = new YouTubeService(settings.Application, settings.DeveloperKey);
             }
             else
             {
@@ -1303,7 +1340,7 @@ namespace Google.YouTube
                 v.YouTubeEntry.Comments != null &&
                 v.YouTubeEntry.Comments.FeedLink != null)
             {
-                Uri target = new Uri(v.YouTubeEntry.Comments.FeedLink.Href);
+                Uri target = CreateUri(v.YouTubeEntry.Comments.FeedLink.Href);
                 rc = new Comment();
                 rc.AtomEntry = this.Service.Insert(target, c.AtomEntry);
             }
@@ -1326,7 +1363,7 @@ namespace Google.YouTube
                 p.PlaylistsEntry.Content != null &&
                 p.PlaylistsEntry.Content.Src != null)
             {
-                Uri target = new Uri(p.PlaylistsEntry.Content.Src.Content);
+                Uri target = CreateUri(p.PlaylistsEntry.Content.Src.Content);
                 newMember = new PlayListMember();
                 newMember.AtomEntry = this.Service.Insert(target, m.AtomEntry);
             }
@@ -1359,11 +1396,23 @@ namespace Google.YouTube
 
                 if (videos.Count > 0)
                 {
-                    meta = this.Batch(videos, new Uri(YouTubeQuery.BatchVideoUri), GDataBatchOperationType.query);
+                    meta = this.Batch(videos, CreateUri(YouTubeQuery.BatchVideoUri), GDataBatchOperationType.query);
                 }
             }
 
             return meta == null? new Feed<Video>(null) : meta ; 
         }
+
+
+        /// <summary>
+        /// returns a single Video (the first) from that stream. Usefull to parse insert/update 
+        /// response streams
+        /// </summary>
+        /// <param name="inputStream"></param>
+        /// <returns></returns>
+        public Video ParseVideo(Stream inputStream)
+        {
+            return ParseEntry<Video>(inputStream, new Uri(YouTubeQuery.DefaultVideoUri));
+        }
     }
 }
